@@ -6,14 +6,12 @@
     Build epub files.
     Originally derived from qthelp.py.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import re
-import sys
-import time
 import codecs
 import zipfile
 from os import path
@@ -30,6 +28,7 @@ from docutils import nodes
 
 from sphinx import addnodes
 from sphinx.builders.html import StandaloneHTMLBuilder
+from sphinx.util.i18n import format_date
 from sphinx.util.osutil import ensuredir, copyfile, EEXIST
 from sphinx.util.smartypants import sphinx_smarty_pants as ssp
 from sphinx.util.console import brown
@@ -41,9 +40,9 @@ from sphinx.util.console import brown
 # output but that may be customized by (re-)setting module attributes,
 # e.g. from conf.py.
 
-_mimetype_template = 'application/epub+zip' # no EOL!
+MIMETYPE_TEMPLATE = 'application/epub+zip'  # no EOL!
 
-_container_template = '''\
+CONTAINER_TEMPLATE = u'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <container version="1.0"
       xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
@@ -54,7 +53,7 @@ _container_template = '''\
 </container>
 '''
 
-_toc_template = '''\
+TOC_TEMPLATE = u'''\
 <?xml version="1.0"?>
 <ncx version="2005-1" xmlns="http://www.daisy.org/z3986/2005/ncx/">
   <head>
@@ -72,7 +71,7 @@ _toc_template = '''\
 </ncx>
 '''
 
-_navpoint_template = '''\
+NAVPOINT_TEMPLATE = u'''\
 %(indent)s  <navPoint id="%(navpoint)s" playOrder="%(playorder)d">
 %(indent)s    <navLabel>
 %(indent)s      <text>%(text)s</text>
@@ -80,10 +79,10 @@ _navpoint_template = '''\
 %(indent)s    <content src="%(refuri)s" />
 %(indent)s  </navPoint>'''
 
-_navpoint_indent = '  '
-_navPoint_template = 'navPoint%d'
+NAVPOINT_INDENT = '  '
+NODE_NAVPOINT_TEMPLATE = 'navPoint%d'
 
-_content_template = '''\
+CONTENT_TEMPLATE = u'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <package xmlns="http://www.idpf.org/2007/opf" version="2.0"
       unique-identifier="%(uid)s">
@@ -110,40 +109,40 @@ _content_template = '''\
 </package>
 '''
 
-_cover_template = '''\
+COVER_TEMPLATE = u'''\
     <meta name="cover" content="%(cover)s"/>
 '''
 
-_coverpage_name = 'epub-cover.html'
+COVERPAGE_NAME = u'epub-cover.html'
 
-_file_template = '''\
+FILE_TEMPLATE = u'''\
     <item id="%(id)s"
           href="%(href)s"
           media-type="%(media_type)s" />'''
 
-_spine_template = '''\
+SPINE_TEMPLATE = u'''\
     <itemref idref="%(idref)s" />'''
 
-_guide_template = '''\
+GUIDE_TEMPLATE = u'''\
     <reference type="%(type)s" title="%(title)s" href="%(uri)s" />'''
 
-_toctree_template = 'toctree-l%d'
+TOCTREE_TEMPLATE = u'toctree-l%d'
 
-_link_target_template = ' [%(uri)s]'
+LINK_TARGET_TEMPLATE = u' [%(uri)s]'
 
-_footnote_label_template = '#%d'
+FOOTNOTE_LABEL_TEMPLATE = u'#%d'
 
-_footnotes_rubric_name = 'Footnotes'
+FOOTNOTES_RUBRIC_NAME = u'Footnotes'
 
-_css_link_target_class = 'link-target'
+CSS_LINK_TARGET_CLASS = u'link-target'
 
 # XXX These strings should be localized according to epub_language
-_guide_titles = {
-    'toc': 'Table of Contents',
-    'cover': 'Cover'
+GUIDE_TITLES = {
+    'toc': u'Table of Contents',
+    'cover': u'Cover'
 }
 
-_media_types = {
+MEDIA_TYPES = {
     '.html': 'application/xhtml+xml',
     '.css': 'text/css',
     '.png': 'image/png',
@@ -155,12 +154,12 @@ _media_types = {
     '.ttf': 'application/x-font-ttf',
 }
 
-_vector_graphics_extensions = ('.svg',)
+VECTOR_GRAPHICS_EXTENSIONS = ('.svg',)
 
 # Regular expression to match colons only in local fragment identifiers.
 # If the URI contains a colon before the #,
 # it is an external link that should not change.
-_refuri_re = re.compile("([^#:]*#)(.*)")
+REFURI_RE = re.compile("([^#:]*#)(.*)")
 
 
 # The epub publisher
@@ -185,6 +184,25 @@ class EpubBuilder(StandaloneHTMLBuilder):
     # don't add sidebar etc.
     embedded = True
 
+    mimetype_template = MIMETYPE_TEMPLATE
+    container_template = CONTAINER_TEMPLATE
+    toc_template = TOC_TEMPLATE
+    navpoint_template = NAVPOINT_TEMPLATE
+    navpoint_indent = NAVPOINT_INDENT
+    node_navpoint_template = NODE_NAVPOINT_TEMPLATE
+    content_template = CONTENT_TEMPLATE
+    cover_template = COVER_TEMPLATE
+    coverpage_name = COVERPAGE_NAME
+    file_template = FILE_TEMPLATE
+    spine_template = SPINE_TEMPLATE
+    guide_template = GUIDE_TEMPLATE
+    toctree_template = TOCTREE_TEMPLATE
+    link_target_template = LINK_TARGET_TEMPLATE
+    css_link_target_class = CSS_LINK_TARGET_CLASS
+    guide_titles = GUIDE_TITLES
+    media_types = MEDIA_TYPES
+    refuri_re = REFURI_RE
+
     def init(self):
         StandaloneHTMLBuilder.init(self)
         # the output files for epub must be .html only
@@ -222,11 +240,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if isinstance(doctree, nodes.reference) and 'refuri' in doctree:
             refuri = doctree['refuri']
             if refuri.startswith('http://') or refuri.startswith('https://') \
-                or refuri.startswith('irc:') or refuri.startswith('mailto:'):
+               or refuri.startswith('irc:') or refuri.startswith('mailto:'):
                 return result
             classes = doctree.parent.attributes['classes']
-            for level in range(8, 0, -1): # or range(1, 8)?
-                if (_toctree_template % level) in classes:
+            for level in range(8, 0, -1):  # or range(1, 8)?
+                if (self.toctree_template % level) in classes:
                     result.append({
                         'level': level,
                         'refuri': self.esc(refuri),
@@ -243,11 +261,12 @@ class EpubBuilder(StandaloneHTMLBuilder):
         and pre and post files not managed by sphinx.
         """
         doctree = self.env.get_and_resolve_doctree(self.config.master_doc,
-            self, prune_toctrees=False, includehidden=True)
+                                                   self, prune_toctrees=False,
+                                                   includehidden=True)
         self.refnodes = self.get_refnodes(doctree, [])
         master_dir = path.dirname(self.config.master_doc)
         if master_dir:
-            master_dir += '/' # XXX or os.sep?
+            master_dir += '/'  # XXX or os.sep?
             for item in self.refnodes:
                 item['refuri'] = master_dir + item['refuri']
         self.toc_add_files(self.refnodes)
@@ -259,7 +278,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
             'level': 1,
             'refuri': self.esc(self.config.master_doc + '.html'),
             'text': ssp(self.esc(
-                    self.env.titles[self.config.master_doc].astext()))
+                self.env.titles[self.config.master_doc].astext()))
         })
         for file, text in reversed(self.config.epub_pre_files):
             refnodes.insert(0, {
@@ -286,7 +305,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         for node in tree.traverse(nodes.reference):
             if 'refuri' in node:
-                m = _refuri_re.match(node['refuri'])
+                m = self.refuri_re.match(node['refuri'])
                 if m:
                     node['refuri'] = self.fix_fragment(m.group(1), m.group(2))
             if 'refid' in node:
@@ -332,11 +351,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 return fn.parent, fn.parent.index(fn) + 1
             for node in tree.traverse(nodes.rubric):
                 if len(node.children) == 1 and \
-                        node.children[0].astext() == _footnotes_rubric_name:
+                        node.children[0].astext() == FOOTNOTES_RUBRIC_NAME:
                     return node.parent, node.parent.index(node) + 1
             doc = tree.traverse(nodes.document)[0]
             rub = nodes.rubric()
-            rub.append(nodes.Text(_footnotes_rubric_name))
+            rub.append(nodes.Text(FOOTNOTES_RUBRIC_NAME))
             doc.append(rub)
             return doc, doc.index(rub) + 1
 
@@ -352,12 +371,12 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     uri.startswith('ftp:')) and uri not in node.astext():
                 idx = node.parent.index(node) + 1
                 if show_urls == 'inline':
-                    uri = _link_target_template % {'uri': uri}
+                    uri = self.link_target_template % {'uri': uri}
                     link = nodes.inline(uri, uri)
-                    link['classes'].append(_css_link_target_class)
+                    link['classes'].append(self.css_link_target_class)
                     node.parent.insert(idx, link)
                 elif show_urls == 'footnote':
-                    label = _footnote_label_template % nr
+                    label = FOOTNOTE_LABEL_TEMPLATE % nr
                     nr += 1
                     footnote_ref = make_footnote_ref(doc, label)
                     node.parent.insert(idx, footnote_ref)
@@ -382,32 +401,32 @@ class EpubBuilder(StandaloneHTMLBuilder):
         # XXX: modifies tree inline
         # Logic modeled from themes/basic/genindex.html
         for key, columns in tree:
-            for entryname, (links, subitems) in columns:
+            for entryname, (links, subitems, key_) in columns:
                 for (i, (ismain, link)) in enumerate(links):
-                    m = _refuri_re.match(link)
+                    m = self.refuri_re.match(link)
                     if m:
                         links[i] = (ismain,
                                     self.fix_fragment(m.group(1), m.group(2)))
                 for subentryname, subentrylinks in subitems:
                     for (i, (ismain, link)) in enumerate(subentrylinks):
-                        m = _refuri_re.match(link)
+                        m = self.refuri_re.match(link)
                         if m:
                             subentrylinks[i] = (ismain,
-                                self.fix_fragment(m.group(1), m.group(2)))
+                                                self.fix_fragment(m.group(1), m.group(2)))
 
     def is_vector_graphics(self, filename):
         """Does the filename extension indicate a vector graphic format?"""
         ext = path.splitext(filename)[-1]
-        return ext in _vector_graphics_extensions
+        return ext in VECTOR_GRAPHICS_EXTENSIONS
 
     def copy_image_files_pil(self):
         """Copy images using the PIL.
         The method tries to read and write the files with the PIL,
         converting the format and resizing the image if necessary/possible.
         """
-        ensuredir(path.join(self.outdir, '_images'))
-        for src in self.status_iterator(self.images, 'copying images... ',
-                                        brown, len(self.images)):
+        ensuredir(path.join(self.outdir, self.imagedir))
+        for src in self.app.status_iterator(self.images, 'copying images... ',
+                                            brown, len(self.images)):
             dest = self.images[src]
             try:
                 img = Image.open(path.join(self.srcdir, src))
@@ -417,7 +436,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                               (path.join(self.srcdir, src), ))
                 try:
                     copyfile(path.join(self.srcdir, src),
-                             path.join(self.outdir, '_images', dest))
+                             path.join(self.outdir, self.imagedir, dest))
                 except (IOError, OSError) as err:
                     self.warn('cannot copy image file %r: %s' %
                               (path.join(self.srcdir, src), err))
@@ -433,7 +452,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     nh = (height * nw) / width
                     img = img.resize((nw, nh), Image.BICUBIC)
             try:
-                img.save(path.join(self.outdir, '_images', dest))
+                img.save(path.join(self.outdir, self.imagedir, dest))
             except (IOError, OSError) as err:
                 self.warn('cannot write image file %r: %s' %
                           (path.join(self.srcdir, src), err))
@@ -462,8 +481,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if pagename.startswith('genindex'):
             self.fix_genindex(addctx['genindexentries'])
         StandaloneHTMLBuilder.handle_page(self, pagename, addctx, templatename,
-            outfilename, event_arg)
-
+                                          outfilename, event_arg)
 
     # Finish by building the epub file
     def handle_finish(self):
@@ -480,7 +498,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.info('writing %s file...' % outname)
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_mimetype_template)
+            f.write(self.mimetype_template)
         finally:
             f.close()
 
@@ -495,7 +513,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 raise
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_container_template)
+            f.write(self.container_template)
         finally:
             f.close()
 
@@ -512,7 +530,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata['copyright'] = self.esc(self.config.epub_copyright)
         metadata['scheme'] = self.esc(self.config.epub_scheme)
         metadata['id'] = self.esc(self.config.epub_identifier)
-        metadata['date'] = self.esc(time.strftime('%Y-%m-%d'))
+        metadata['date'] = self.esc(format_date('%Y-%m-%d', language=self.config.language,
+                                                warn=self.warn))
         metadata['files'] = files
         metadata['spine'] = spine
         metadata['guide'] = guide
@@ -530,9 +549,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
         olen = len(outdir)
         projectfiles = []
         self.files = []
-        self.ignored_files = ['.buildinfo',
-            'mimetype', 'content.opf', 'toc.ncx', 'META-INF/container.xml',
-            self.config.epub_basename + '.epub'] + \
+        self.ignored_files = ['.buildinfo', 'mimetype', 'content.opf',
+                              'toc.ncx', 'META-INF/container.xml',
+                              self.config.epub_basename + '.epub'] + \
             self.config.epub_exclude_files
         for root, dirs, files in os.walk(outdir):
             for fn in files:
@@ -540,17 +559,17 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 if filename in self.ignored_files:
                     continue
                 ext = path.splitext(filename)[-1]
-                if ext not in _media_types:
+                if ext not in self.media_types:
                     # we always have JS and potentially OpenSearch files, don't
                     # always warn about them
                     if ext not in ('.js', '.xml'):
                         self.warn('unknown mimetype for %s, ignoring' % filename)
                     continue
                 filename = filename.replace(os.sep, '/')
-                projectfiles.append(_file_template % {
+                projectfiles.append(self.file_template % {
                     'href': self.esc(filename),
                     'id': self.esc(self.make_id(filename)),
-                    'media_type': self.esc(_media_types[ext])
+                    'media_type': self.esc(self.media_types[ext])
                 })
                 self.files.append(filename)
 
@@ -561,43 +580,43 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 continue
             if item['refuri'] in self.ignored_files:
                 continue
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id(item['refuri']))
             })
         for info in self.domain_indices:
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id(info[0] + self.out_suffix))
             })
         if self.get_builder_config('use_index', 'epub'):
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id('genindex' + self.out_suffix))
             })
 
         # add the optional cover
-        content_tmpl = _content_template
+        content_tmpl = self.content_template
         html_tmpl = None
         if self.config.epub_cover:
             image, html_tmpl = self.config.epub_cover
             image = image.replace(os.sep, '/')
             mpos = content_tmpl.rfind('</metadata>')
-            cpos = content_tmpl.rfind('\n', 0 , mpos) + 1
+            cpos = content_tmpl.rfind('\n', 0, mpos) + 1
             content_tmpl = content_tmpl[:cpos] + \
-                _cover_template % {'cover': self.esc(self.make_id(image))} + \
+                COVER_TEMPLATE % {'cover': self.esc(self.make_id(image))} + \
                 content_tmpl[cpos:]
             if html_tmpl:
-                spine.insert(0, _spine_template % {
-                    'idref': self.esc(self.make_id(_coverpage_name))})
-                if _coverpage_name not in self.files:
-                    ext = path.splitext(_coverpage_name)[-1]
-                    self.files.append(_coverpage_name)
-                    projectfiles.append(_file_template % {
-                        'href': self.esc(_coverpage_name),
-                        'id': self.esc(self.make_id(_coverpage_name)),
-                        'media_type': self.esc(_media_types[ext])
+                spine.insert(0, self.spine_template % {
+                    'idref': self.esc(self.make_id(self.coverpage_name))})
+                if self.coverpage_name not in self.files:
+                    ext = path.splitext(self.coverpage_name)[-1]
+                    self.files.append(self.coverpage_name)
+                    projectfiles.append(self.file_template % {
+                        'href': self.esc(self.coverpage_name),
+                        'id': self.esc(self.make_id(self.coverpage_name)),
+                        'media_type': self.esc(self.media_types[ext])
                     })
                 ctx = {'image': self.esc(image), 'title': self.config.project}
                 self.handle_page(
-                        path.splitext(_coverpage_name)[0], ctx, html_tmpl)
+                    path.splitext(self.coverpage_name)[0], ctx, html_tmpl)
 
         guide = []
         auto_add_cover = True
@@ -611,23 +630,23 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     auto_add_cover = False
                 if type == 'toc':
                     auto_add_toc = False
-                guide.append(_guide_template % {
-                        'type': self.esc(type),
-                        'title': self.esc(title),
-                        'uri': self.esc(uri)
-                        })
+                guide.append(self.guide_template % {
+                    'type': self.esc(type),
+                    'title': self.esc(title),
+                    'uri': self.esc(uri)
+                })
         if auto_add_cover and html_tmpl:
-            guide.append(_guide_template % {
-                    'type': 'cover',
-                    'title': _guide_titles['cover'],
-                    'uri': self.esc(_coverpage_name)
-                    })
+            guide.append(self.guide_template % {
+                'type': 'cover',
+                'title': self.guide_titles['cover'],
+                'uri': self.esc(self.coverpage_name)
+            })
         if auto_add_toc and self.refnodes:
-            guide.append(_guide_template % {
-                    'type': 'toc',
-                    'title': _guide_titles['toc'],
-                    'uri': self.esc(self.refnodes[0]['refuri'])
-                    })
+            guide.append(self.guide_template % {
+                'type': 'toc',
+                'title': self.guide_titles['toc'],
+                'uri': self.esc(self.refnodes[0]['refuri'])
+            })
         projectfiles = '\n'.join(projectfiles)
         spine = '\n'.join(spine)
         guide = '\n'.join(guide)
@@ -635,8 +654,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
         # write the project file
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(content_tmpl % \
-                self.content_metadata(projectfiles, spine, guide))
+            f.write(content_tmpl %
+                    self.content_metadata(projectfiles, spine, guide))
         finally:
             f.close()
 
@@ -646,10 +665,10 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if incr:
             self.playorder += 1
         self.tocid += 1
-        node['indent'] = _navpoint_indent * level
-        node['navpoint'] = self.esc(_navPoint_template % self.tocid)
+        node['indent'] = self.navpoint_indent * level
+        node['navpoint'] = self.esc(self.node_navpoint_template % self.tocid)
         node['playorder'] = self.playorder
-        return _navpoint_template % node
+        return self.navpoint_template % node
 
     def insert_subnav(self, node, subnav):
         """Insert nested navpoints for given node.
@@ -720,7 +739,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
 
         if self.config.epub_tocscope == 'default':
             doctree = self.env.get_and_resolve_doctree(self.config.master_doc,
-                self, prune_toctrees=False, includehidden=False)
+                                                       self, prune_toctrees=False,
+                                                       includehidden=False)
             refnodes = self.get_refnodes(doctree, [])
             self.toc_add_files(refnodes)
         else:
@@ -731,7 +751,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         level = min(level, self.config.epub_tocdepth)
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_toc_template % self.toc_metadata(level, navpoints))
+            f.write(self.toc_template % self.toc_metadata(level, navpoints))
         finally:
             f.close()
 
@@ -744,18 +764,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.info('writing %s file...' % outname)
         projectfiles = ['META-INF/container.xml', 'content.opf', 'toc.ncx'] \
             + self.files
-        epub = zipfile.ZipFile(path.join(outdir, outname), 'w', \
-            zipfile.ZIP_DEFLATED)
-        epub.write(path.join(outdir, 'mimetype'), 'mimetype', \
-            zipfile.ZIP_STORED)
+        epub = zipfile.ZipFile(path.join(outdir, outname), 'w',
+                               zipfile.ZIP_DEFLATED)
+        epub.write(path.join(outdir, 'mimetype'), 'mimetype',
+                   zipfile.ZIP_STORED)
         for file in projectfiles:
             fp = path.join(outdir, file)
-            if sys.version_info < (2, 6):
-                # When zipile.ZipFile.write call with unicode filename, ZipFile
-                # encode filename to 'utf-8' (only after Python-2.6).
-                if isinstance(file, str):
-                    # OEBPS Container Format (OCF) 2.0.1 specification require
-                    # "File Names MUST be UTF-8 encoded".
-                    file = file.encode('utf-8')
             epub.write(fp, file, zipfile.ZIP_DEFLATED)
         epub.close()
