@@ -120,7 +120,7 @@ def main(argv):
 
     # parse options
     try:
-        opts, args = parser.parse_args(argv[1:])
+        opts, args = parser.parse_args(list(argv[1:]))
     except SystemExit as err:
         return err.code
 
