@@ -16,20 +16,23 @@ from six import string_types, text_type
 from sphinx.transforms import ApplySourceWorkaround, ExtraTranslatableNodes, Locale, \
     CitationReferences, DefaultSubstitutions, MoveModuleTargets, HandleCodeBlocks, \
     AutoNumbering, SortIds, RemoveTranslatableInline
-from sphinx.util import import_object
+from sphinx.util import import_object, split_docinfo
 
 
 class SphinxBaseReader(standalone.Reader):
     """
     Add our source parsers
     """
-    def __init__(self, parsers={}, *args, **kwargs):
+    def __init__(self, app, parsers={}, *args, **kwargs):
         standalone.Reader.__init__(self, *args, **kwargs)
         self.parser_map = {}
         for suffix, parser_class in parsers.items():
             if isinstance(parser_class, string_types):
                 parser_class = import_object(parser_class, 'source parser')
-            self.parser_map[suffix] = parser_class()
+            parser = parser_class()
+            if hasattr(parser, 'set_application'):
+                parser.set_application(app)
+            self.parser_map[suffix] = parser
 
     def read(self, source, parser, settings):
         self.source = source
@@ -114,8 +117,9 @@ class SphinxFileInput(FileInput):
             arg = [data]
             self.app.emit('source-read', self.env.docname, arg)
             data = arg[0]
+        docinfo, data = split_docinfo(data)
         if self.env.config.rst_epilog:
             data = data + '\n' + self.env.config.rst_epilog + '\n'
         if self.env.config.rst_prolog:
             data = self.env.config.rst_prolog + '\n' + data
-        return data
+        return docinfo + data
