@@ -57,13 +57,16 @@ DOUBLESTAREQUAL = 47
 DOUBLESLASH = 48
 DOUBLESLASHEQUAL = 49
 AT = 50
-OP = 51
-COMMENT = 52
-NL = 53
-RARROW = 54
-ERRORTOKEN = 55
-ELLIPSIS = 56
-N_TOKENS = 57
+ATEQUAL = 51
+RARROW = 52
+ELLIPSIS = 53
+OP = 54
+AWAIT = 55
+ASYNC = 56
+COMMENT = 57
+NL = 58
+ERRORTOKEN = 59
+N_TOKENS = 60
 NT_OFFSET = 256
 #--end constants--
 
