@@ -5,9 +5,10 @@
 
     Operating system-related utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import os
 import re
@@ -16,8 +17,10 @@ import time
 import errno
 import locale
 import shutil
-import gettext
 from os import path
+import contextlib
+
+from six import PY2, text_type
 
 # Errnos that we need.
 EEXIST = getattr(errno, 'EEXIST', 0)
@@ -32,10 +35,16 @@ EINVAL = getattr(errno, 'EINVAL', 0)
 # hangover from more *nix-oriented origins.
 SEP = "/"
 
+
 def os_path(canonicalpath):
     return canonicalpath.replace(SEP, path.sep)
 
 
+def canon_path(nativepath):
+    """Return path in OS-independent form"""
+    return nativepath.replace(path.sep, SEP)
+
+
 def relative_uri(base, to):
     """Return a relative URL from ``base`` to ``to``."""
     if to.startswith(SEP):
@@ -55,7 +64,7 @@ def relative_uri(base, to):
     if len(b2) == 1 and t2 == ['']:
         # Special case: relative_uri('f/index.html','f/') should
         # return './', not ''
-        return '.' +  SEP
+        return '.' + SEP
     return ('..' + SEP) * (len(b2)-1) + SEP.join(t2)
 
 
@@ -69,6 +78,9 @@ def ensuredir(path):
             raise
 
 
+# This function is same as os.walk of Python2.6, 2.7, 3.2, 3.3 except a
+# customization that check UnicodeError.
+# The customization obstacle to replace the function with the os.walk.
 def walk(top, topdown=True, followlinks=False):
     """Backport of os.walk from 2.6, where the *followlinks* argument was
     added.
@@ -80,9 +92,9 @@ def walk(top, topdown=True, followlinks=False):
         try:
             fullpath = path.join(top, name)
         except UnicodeError:
-            print((
-                '%s:: ERROR: non-ASCII filename not supported on this '
-                'filesystem encoding %r, skipped.' % (name, fs_encoding)), file=sys.stderr)
+            print('%s:: ERROR: non-ASCII filename not supported on this '
+                  'filesystem encoding %r, skipped.' % (name, fs_encoding),
+                  file=sys.stderr)
             continue
         if path.isdir(fullpath):
             dirs.append(name)
@@ -140,59 +152,64 @@ def copyfile(source, dest):
 
 no_fn_re = re.compile(r'[^a-zA-Z0-9_-]')
 
+
 def make_filename(string):
     return no_fn_re.sub('', string) or 'sphinx'
 
-if sys.version_info < (3, 0):
-    # strftime for unicode strings
-    def ustrftime(format, *args):
+
+def ustrftime(format, *args):
+    # [DEPRECATED] strftime for unicode strings
+    # It will be removed at Sphinx-1.5
+    if not args:
+        # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
+        # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
+        source_date_epoch = os.getenv('SOURCE_DATE_EPOCH')
+        if source_date_epoch is not None:
+            time_struct = time.gmtime(float(source_date_epoch))
+            args = [time_struct]
+    if PY2:
         # if a locale is set, the time strings are encoded in the encoding
         # given by LC_TIME; if that is available, use it
         enc = locale.getlocale(locale.LC_TIME)[1] or 'utf-8'
-        return time.strftime(str(format).encode(enc), *args).decode(enc)
-else:
-    ustrftime = time.strftime
+        return time.strftime(text_type(format).encode(enc), *args).decode(enc)
+    else:  # Py3
+        # On Windows, time.strftime() and Unicode characters will raise UnicodeEncodeError.
+        # http://bugs.python.org/issue8304
+        try:
+            return time.strftime(format, *args)
+        except UnicodeEncodeError:
+            r = time.strftime(format.encode('unicode-escape').decode(), *args)
+            return r.encode().decode('unicode-escape')
 
 
 def safe_relpath(path, start=None):
-    from sphinx.util.pycompat import relpath
     try:
-        return relpath(path, start)
+        return os.path.relpath(path, start)
     except ValueError:
         return path
 
-def find_catalog(docname, compaction):
-    if compaction:
-        ret = docname.split(SEP, 1)[0]
-    else:
-        ret = docname
-
-    return ret
-
-
-def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
-    from sphinx.util.pycompat import relpath
-    if not(lang and locale_dirs):
-        return []
-
-    domain = find_catalog(docname, compaction)
-    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])
-             for dir_ in locale_dirs]
-    files = [relpath(f, srcdir) for f in files if f]
-    return files
-
 
 fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
 
 
-if sys.version_info < (3, 0):
-    bytes = str
-else:
-    bytes = bytes
-
-
 def abspath(pathdir):
     pathdir = path.abspath(pathdir)
     if isinstance(pathdir, bytes):
         pathdir = pathdir.decode(fs_encoding)
     return pathdir
+
+
+def getcwd():
+    if hasattr(os, 'getcwdu'):
+        return os.getcwdu()
+    return os.getcwd()
+
+
+@contextlib.contextmanager
+def cd(target_dir):
+    cwd = getcwd()
+    try:
+        os.chdir(target_dir)
+        yield
+    finally:
+        os.chdir(cwd)
